@@ -6,7 +6,8 @@ import sys
 if sys.version_info >= (3, 8):
     from typing import TypedDict, Union
 else:
-    from typing_extensions import TypeDict, Union
+    from typing_extensions import TypedDict
+    from typing import Union
 
 from requests.sessions import Request
 from passageidentity.helper import fetchApp, getAuthTokenFromRequest
@@ -41,11 +42,25 @@ class Passage():
         url: str
         ttl: int
 
+    class PassageAppType(TypedDict):
+        name: str
+        id: str
+        auth_origin: str
+        redirect_url: str
+        login_url: str
+        rsa_public_key: str
+        allowed_identifier: str
+        require_identifier_verification: bool
+        session_timeout_length: int
+        user_metadata_schema: list
+        layouts: list
+
     class PassageUserType(TypedDict):
         created_at: str
         updated_at: str
         status: UserStatus
         email_verified: bool
+        phone_verified: bool
         email: str
         phone: str
         id: str
@@ -58,6 +73,7 @@ class Passage():
     class PassageDeviceType(TypedDict):
         created_at: str
         updated_at: str
+        last_login_at: str
         id: str
         cred_id: str
         friendly_name: str
@@ -183,6 +199,14 @@ class Passage():
         except Exception as e:
             raise PassageError("Could not create magic link")
 
+    """
+    Use Passage API to get info for their app.
+    """
+
+    def getApp(self) -> Union[PassageAppType, PassageError]:
+        app_info = fetchApp(self.app_id)
+        return PassageApp(app_info)
+
 
     """
     Use Passage API to get info for a user, look up by user ID
@@ -366,6 +390,19 @@ class Passage():
         except Exception as e:
             raise PassageError("Could not create user")
 
+class PassageApp:
+    def __init__(self, fields={}):
+        self.name = fields["name"]
+        self.id = fields["id"]
+        self.auth_origin = fields["auth_origin"]
+        self.redirect_url = fields["redirect_url"]
+        self.login_url = fields["login_url"]
+        self.rsa_public_key = fields["rsa_public_key"]
+        self.allowed_identifier = fields["allowed_identifier"]
+        self.require_identifier_verification = fields["require_identifier_verification"]
+        self.session_timeout_length = fields["session_timeout_length"]
+        self.user_metadata_schema = fields["user_metadata_schema"]
+        self.layouts = fields["layouts"]
 class PassageDevice:
     def __init__(self, fields={}):
         self.id = fields["id"]
@@ -377,6 +414,10 @@ class PassageDevice:
             self.updated_at = datetime.strptime(time_to_milliseconds(fields["updated_at"]),"%Y-%m-%dT%H:%M:%S.%fZ")
         except:
             self.updated_at = datetime.strptime(fields["updated_at"],"%Y-%m-%dT%H:%M:%SZ")
+        try:
+            self.last_login_at = datetime.strptime(time_to_milliseconds(fields["last_login_at"]),"%Y-%m-%dT%H:%M:%S.%fZ")
+        except:
+            self.last_login_at = datetime.strptime(fields["last_login_at"],"%Y-%m-%dT%H:%M:%SZ")
         self.friendly_name = fields["friendly_name"]
         self.usage_count = fields["usage_count"]
 
@@ -400,6 +441,7 @@ class PassageUser:
         self.phone = fields["phone"]
         self.status = fields["status"]
         self.email_verified = fields["email_verified"]
+        self.phone_verified = fields["phone_verified"]
         self.user_metadata = fields["user_metadata"]
         try:
             self.created_at = datetime.strptime(time_to_milliseconds(fields["created_at"]),"%Y-%m-%dT%H:%M:%S.%fZ")
@@ -443,4 +485,4 @@ def time_to_milliseconds(timeString: str) -> str:
 		return timeString
 	
 	# ensure 6 decimal places, add back '.' and 'Z' to string
-	return time[0] + "." + decimalNumbers[:6] + "Z"
\ No newline at end of file
+	return time[0] + "." + decimalNumbers[:6] + "Z"
